@@ -27,6 +27,7 @@
 
 #include "e-cal-backend-exchange.h"
 #include "e2k-cal-utils.h"
+#include "e2k-uri.h"
 
 #include "e2k-propnames.h"
 #include "e2k-restriction.h"
@@ -48,12 +49,15 @@ struct ECalBackendExchangePrivate {
 	/* Timezones */
 	GHashTable *timezones;
 	icaltimezone *default_timezone;
+#ifdef OFFLINE_SUPPORTED
+	CalMode mode;
+#endif
 };
 
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
 static GObjectClass *parent_class = NULL;
 
-#define d(x)
+#define d(x) (x)
 
 static ECalBackendSyncStatus
 is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean *read_only)
@@ -71,10 +75,15 @@ get_cal_address (ECalBackendSync *backend, EDataCal *cal, char **address)
 {
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	ExchangeHierarchy *hier;
-
-	hier = e_folder_exchange_get_hierarchy (cbex->folder);
-	d(printf("ecbe_get_cal_address(%p, %p) -> %s\n", backend, cal, hier->owner_email));
-	*address = g_strdup (hier->owner_email);
+#ifdef OFFLINE_SUPPORTED
+	if (cbex->priv->mode == CAL_MODE_REMOTE) {
+#endif
+		hier = e_folder_exchange_get_hierarchy (cbex->folder);
+		d(printf("ecbe_get_cal_address(%p, %p) -> %s\n", backend, cal, hier->owner_email));
+#ifdef OFFLINE_SUPPORTED
+	}
+#endif
+	*address = g_strdup (hier->owner_email); /* FIXME : For offline, fetch this from gconf */
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -116,16 +125,24 @@ get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabil
 }
 
 static void
-load_cache (ECalBackendExchange *cbex)
+load_cache (ECalBackendExchange *cbex, E2kUri *e2kuri)
 {
 	icalcomponent *vcalcomp, *comp, *tmp_comp;
 	struct icaltimetype comp_last_mod, folder_last_mod;
 	icalcomponent_kind kind;
 	icalproperty *prop;
 	char *lastmod;
-
-	cbex->priv->object_cache_file =
-		e_folder_exchange_get_storage_file (cbex->folder, "cache.ics");
+#ifdef OFFLINE_SUPPORTED
+	if (cbex->priv->mode == CAL_MODE_REMOTE) {
+#endif
+		cbex->priv->object_cache_file =
+			e_folder_exchange_get_storage_file (cbex->folder, "cache.ics");
+#ifdef OFFLINE_SUPPORTED
+	} else {
+		cbex->priv->object_cache_file = 
+			exchange_offline_build_object_cache_file (e2kuri, "cache.ics", TRUE);
+	}
+#endif
 
 	vcalcomp = e_cal_util_parse_ics_file (cbex->priv->object_cache_file);
 	if (!vcalcomp)
@@ -184,7 +201,7 @@ save_object (gpointer key, gpointer value, gpointer vcalcomp)
 	icalcomponent *icalcomp;
 	GList *l;
 
-	icalcomp = icalcomponent_new_clone (ecomp->comp);
+	icalcomp = icalcomponent_new_clone (ecomp->icomp);
 	icalcomponent_add_component (vcalcomp, icalcomp);
 
 	for (l = ecomp->instances; l; l = l->next) {
@@ -213,20 +230,17 @@ timeout_save_cache (gpointer user_data)
 
 	tmpfile = g_strdup_printf ("%s~", cbex->priv->object_cache_file);
 	f = fopen (tmpfile, "w");
-	if (!f) {
-		g_free (tmpfile);
-		return FALSE;
-	}
+	if (!f)
+		goto error;
 
 	len = strlen (data);
 	nwrote = fwrite (data, 1, len, f);
-	if (fclose (f) != 0 || nwrote != len) {
-		g_free (tmpfile);
-		return FALSE;
-	}
+	if (fclose (f) != 0 || nwrote != len)
+		goto error;
 
 	if (rename (tmpfile, cbex->priv->object_cache_file) != 0)
 		unlink (tmpfile);
+error:
 	g_free (tmpfile);
 	return FALSE;
 }
@@ -254,13 +268,24 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 	const char *prop = PR_ACCESS;
 	E2kHTTPStatus status;
 	E2kResult *results;
+	E2kUri *euri;
 	int nresults;
 	guint access = 0;
 
 	d(printf("ecbe_open_calendar(%p, %p, %sonly if exists, user=%s, pass=%s)\n", backend, cal, only_if_exists?"":"not ", username?username:"(null)", password?password:"(null)"));
 
-	/* Make sure we have an open connection */
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
+	euri = e2k_uri_new (uristr);
+#ifdef OFFLINE_SUPPORTED
+	if (cbex->priv->mode == CAL_MODE_LOCAL) {
+		printf ("ECBE : cal is offline .. load cache\n");
+		load_cache (cbex, euri);
+		e2k_uri_free (euri);
+		return GNOME_Evolution_Calendar_Success;
+	}
+#endif		
+		
+	/* Make sure we have an open connection */
 	cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
 	if (!cbex->account)
 		return GNOME_Evolution_Calendar_RepositoryOffline;
@@ -298,7 +323,7 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 
 	cbex->priv->read_only = ((access & MAPI_ACCESS_CREATE_CONTENTS) == 0);
 
-	load_cache (cbex);
+	load_cache (cbex, euri);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -307,7 +332,9 @@ static ECalBackendSyncStatus
 remove_calendar (ECalBackendSync *backend, EDataCal *cal)
 {
 	d(printf("ecbe_remove_calendar(%p, %p)\n", backend, cal));
-	return GNOME_Evolution_Calendar_PermissionDenied;
+
+	//display_error_dialog("Can not perform this operation, Use Exchange component for folder operations\n");
+	return GNOME_Evolution_Calendar_PermissionDenied; /* Error code is not handled in Evolution */
 
 	/* FIXME: Deleting calendar/tasks from respective views */
 #if 0
@@ -408,16 +435,15 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 	ECalBackendExchangeComponent *ecomp;
 	gboolean is_instance;
 	const char *uid;
-	struct icaltimetype rid;
 
-	d(printf("ecbe_add_object(%p, %s, %s)\n", cbex, href, lastmod));
+	printf("ecbe_add_object(%p, %s, %s)\n", cbex, href, lastmod);
 
 	uid = icalcomponent_get_uid (comp);
 	ecomp = g_hash_table_lookup (cbex->priv->objects, uid);
 
 	is_instance = (icalcomponent_get_first_property (comp, ICAL_RECURRENCEID_PROPERTY) != NULL);
 
-	if (ecomp && ecomp->comp && !is_instance)
+	if (ecomp && ecomp->icomp && !is_instance)
 		return FALSE;
 
 	if (!ecomp) {
@@ -436,25 +462,10 @@ e_cal_backend_exchange_add_object (ECalBackendExchange *cbex,
 	}
 
 	if (is_instance) {
-		GList *l;
-		struct icaltimetype inst_rid;
-		gboolean inst_found = FALSE;
-
-		rid = icalcomponent_get_recurrenceid (comp);
-		for (l = ecomp->instances; l; l = l->next) {
-			inst_rid = icalcomponent_get_recurrenceid (l->data);
-			if (icaltime_compare (inst_rid, rid) == 0) {
-				inst_found = TRUE;
-				break;
-			}
-		}
-		if (!inst_found) {
-
-			ecomp->instances = g_list_prepend (ecomp->instances,
+		ecomp->instances = g_list_prepend (ecomp->instances,
 						   icalcomponent_new_clone (comp));
-		}
 	} else
-		ecomp->comp = icalcomponent_new_clone (comp);
+		ecomp->icomp = icalcomponent_new_clone (comp);
 
 	save_cache (cbex);
 	return TRUE;
@@ -470,8 +481,8 @@ discard_detached_instance (ECalBackendExchangeComponent *ecomp,
 	for (inst = ecomp->instances; inst; inst = inst->next) {
 		inst_rid = icalcomponent_get_recurrenceid (inst->data);
 		if (icaltime_compare (inst_rid, rid) == 0) {
-			ecomp->instances = g_list_remove (ecomp->instances, inst->data);
 			icalcomponent_free (inst->data);
+			ecomp->instances = g_list_remove (ecomp->instances, inst->data);
 			return;
 		}
 	}
@@ -510,8 +521,8 @@ e_cal_backend_exchange_modify_object (ECalBackendExchange *cbex,
 		return FALSE;
 
 	if (mod == CALOBJ_MOD_ALL || icaltime_is_null_time (rid)) {
-		icalcomponent_free (ecomp->comp);
-		ecomp->comp = icalcomponent_new_clone (comp);
+		icalcomponent_free (ecomp->icomp);
+		ecomp->icomp = icalcomponent_new_clone (comp);
 	} else {
 		discard_detached_instance (ecomp, rid);
 		ecomp->instances = g_list_prepend (ecomp->instances,
@@ -564,7 +575,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 /*To be overriden by Calendar and Task classes*/
 static ECalBackendSyncStatus
 modify_object (ECalBackendSync *backend, EDataCal *cal, 
-			const char * calobj, CalObjModType mod, char **old_object)
+			const char * calobj, CalObjModType mod, char **old_object, char **new_object)
 {
 	return GNOME_Evolution_Calendar_OtherError;
 }
@@ -595,20 +606,16 @@ get_object (ECalBackendSync *backend, EDataCal *cal,
 	/*any other asserts?*/
 	
 	ecomp = g_hash_table_lookup (cbex->priv->objects, uid);
-	if ((!ecomp) || (!ecomp->comp))
+	if (!ecomp)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	
 	/*anything on recur id here????*/
 	
 	comp = e_cal_component_new ();
-	if (e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->comp))) {
-		*object = e_cal_component_get_as_string (comp);
-	} else {
-		*object = NULL;
-		g_object_unref (comp);
-		return GNOME_Evolution_Calendar_ObjectNotFound;
-	}
+	e_cal_component_set_icalcomponent (comp, ecomp->icomp);
 
+	*object = e_cal_component_get_as_string (comp);
+	
 	g_object_unref (comp);
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -693,11 +700,14 @@ static ECalBackendSyncStatus
 get_default_object (ECalBackendSync *backend, EDataCal *cal, char **object)
 {
 	icalcomponent *comp;
+	char *ical_obj;
 
 	d(printf("ecbe_get_default_object(%p, %p)\n", backend, cal));
 
 	comp = e_cal_util_new_component (e_cal_backend_get_kind (E_CAL_BACKEND (backend)));
-	*object = g_strdup (icalcomponent_as_ical_string (comp));
+	ical_obj = icalcomponent_as_ical_string (comp);
+	*object = g_strdup (ical_obj);
+
 	icalcomponent_free (comp);
 
 	return GNOME_Evolution_Calendar_Success;
@@ -718,27 +728,21 @@ match_object_sexp (gpointer key, gpointer value, gpointer data)
 	ECalBackendExchangeComponent *ecomp = value;
 	MatchObjectData *match_data = data;
 	ECalComponent *comp;
-	icalcomponent *tmp;
-	char *ecal_str;
 	
 	comp = e_cal_component_new ();
-	
-	tmp = icalcomponent_new_clone (ecomp->comp);
-	if (tmp) {
-		e_cal_component_set_icalcomponent (comp, tmp);
-		if ((!match_data->search_needed) ||
-		    (e_cal_backend_sexp_match_comp (match_data->obj_sexp, comp, match_data->backend))) {
-			ecal_str = e_cal_component_get_as_string (comp);
-			if (ecal_str)
-				match_data->obj_list = g_list_append (match_data->obj_list, ecal_str);
+	e_cal_component_set_icalcomponent (comp, ecomp->icomp);
 
-			#if 0
-			/* match also recurrences */
-			g_hash_table_foreach (obj_data->recurrences,
-					      (GHFunc) match_recurrence_sexp,
-					      match_data);
-			#endif
-		}
+	if ((!match_data->search_needed) ||
+	    (e_cal_backend_sexp_match_comp (match_data->obj_sexp, comp, match_data->backend))) {
+		match_data->obj_list = g_list_append (match_data->obj_list,
+						      e_cal_component_get_as_string (comp));
+
+		#if 0
+		/* match also recurrences */
+		g_hash_table_foreach (obj_data->recurrences,
+				      (GHFunc) match_recurrence_sexp,
+				      match_data);
+		#endif
 	}
 	g_object_unref (comp);
 }
@@ -780,6 +784,7 @@ get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	icaltimezone *zone;
 	icalcomponent *vtzcomp;
+	char *ical_obj;
 
 	d(printf("ecbe_get_timezone(%p, %p, %s)\n", backend, cal, tzid));
 
@@ -791,7 +796,10 @@ get_timezone (ECalBackendSync *backend, EDataCal *cal,
 	if (!vtzcomp)
 		return GNOME_Evolution_Calendar_OtherError;
 
-	*object = g_strdup (icalcomponent_as_ical_string (vtzcomp));
+	ical_obj = icalcomponent_as_ical_string (vtzcomp);
+	*object = g_strdup (ical_obj);
+
+	icalcomponent_free (vtzcomp);
 	return  GNOME_Evolution_Calendar_Success;
 }
 
@@ -890,30 +898,21 @@ match_object (gpointer key, gpointer value, gpointer data)
 	ECalBackendExchangeComponent *ecomp = value;
 	struct search_data *sd = data;
 	ECalComponent *cal_comp;
-	icalcomponent *tmp;
-	char *cal_str;
 	GList *inst;
 
 	/* FIXME: we shouldn't have to convert to ECalComponent here */
 	cal_comp = e_cal_component_new ();
-
-	/*Find a way to test if the icalcomp added to cal_comp is not null */
-	tmp = icalcomponent_new_clone (ecomp->comp);
-	if (tmp) {
-		e_cal_component_set_icalcomponent (cal_comp, tmp);
-		if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
-			cal_str = e_cal_component_get_as_string (cal_comp);
-			if (cal_str)
-				sd->matches = g_list_prepend (sd->matches, cal_str);
-		}
+	e_cal_component_set_icalcomponent (cal_comp, icalcomponent_new_clone (ecomp->icomp));
+	if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
+		sd->matches = g_list_prepend (sd->matches,
+					      e_cal_component_get_as_string (cal_comp));
 	}
 
 	for (inst = ecomp->instances; inst; inst = inst->next) {
 		e_cal_component_set_icalcomponent (cal_comp, icalcomponent_new_clone (inst->data));
 		if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
-			cal_str = e_cal_component_get_as_string (cal_comp);
-			if (cal_str)
-				sd->matches = g_list_prepend (sd->matches, cal_str);
+			sd->matches = g_list_prepend (sd->matches,
+						      e_cal_component_get_as_string (cal_comp));
 		}
 	}
 
@@ -950,28 +949,101 @@ start_query (ECalBackend *backend, EDataCalView *view)
 	e_data_cal_view_notify_done (view, GNOME_Evolution_Calendar_Success);
 }
 
+#ifdef OFFLINE_SUPPORTED
+gboolean 
+e_cal_backend_exchange_is_online (ECalBackendExchange *cbex)
+{
+	if (cbex->priv->mode == CAL_MODE_LOCAL)
+		return FALSE;
+	else
+		return TRUE;
+}
+#endif
+
 static CalMode
 get_mode (ECalBackend *backend)
 {
+#ifdef OFFLINE_SUPPORTED
+	ECalBackendExchange *cbex;
+	ECalBackendExchangePrivate *priv;
+	
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	priv = cbex->priv;
+
 	d(printf("ecbe_get_mode(%p)\n", backend));
 
+	return priv->mode;
+#else
 	return CAL_MODE_REMOTE;
+#endif
 }
 
 static void
 set_mode (ECalBackend *backend, CalMode mode)
 {
-	d(printf("ecbe_set_mode(%p)\n", backend));
+#ifdef OFFLINE_SUPPORTED
+	ECalBackendExchange *cbex;
+	ECalBackendExchangePrivate *priv;
+	
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	priv = cbex->priv;
 
-	if (mode == CAL_MODE_REMOTE) {
+	d(printf("ecbe_set_mode(%p) : mode : %d\n", backend, mode));
+
+	if (priv->mode == mode) {
 		e_cal_backend_notify_mode (
 			backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
+			cal_mode_to_corba (mode));
+	}
+
+	switch (mode) {
+	
+	case CAL_MODE_REMOTE:
+			/* Change status to be online now */
+#if 0
+			if (exchange_account_set_online (cbex->account)) { printf ("mode set to online\n"); 
+				priv->mode = CAL_MODE_REMOTE;
+			}
+			else { printf ("failed to set mode to online\n");
+				break;
+			}
+#endif
+			/* Should we check for access rights before setting this ? */
+			priv->read_only = FALSE;
+			e_cal_backend_notify_mode (backend, 
+				GNOME_Evolution_Calendar_CalListener_MODE_SET,
+				GNOME_Evolution_Calendar_MODE_REMOTE);
+			/* FIXME : Check if online and check if authentication
+				is needed */
+			break;
+
+	case CAL_MODE_LOCAL:
+			/* FIXME : Update the cache before closing the connection */
+			printf ("set mode to offline\n");
+			priv->mode = CAL_MODE_LOCAL;
+			/* FIXME : Set connection to NULL and become offline */
+			exchange_account_set_offline (cbex->account);
+			e_cal_backend_notify_mode (backend, 
+				GNOME_Evolution_Calendar_CalListener_MODE_SET,
+				GNOME_Evolution_Calendar_MODE_LOCAL);
+			break;
+
+	default :
+		e_cal_backend_notify_mode (
+			backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
+			cal_mode_to_corba (mode));
+	}
+#else
+	if (mode == CAL_MODE_REMOTE) {
+		e_cal_backend_notify_mode (
+                        backend, GNOME_Evolution_Calendar_CalListener_MODE_SET,
 			GNOME_Evolution_Calendar_MODE_REMOTE);
 	} else {
-		e_cal_backend_notify_mode (
+		 e_cal_backend_notify_mode (
 			backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED,
 			GNOME_Evolution_Calendar_MODE_REMOTE);
 	}
+#endif
 }
 
 static ECalBackendSyncStatus
@@ -1074,9 +1146,18 @@ e_cal_backend_exchange_get_cal_address (ECalBackendSync *backend)
 	ExchangeHierarchy *hier;
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex), NULL);
-
+#ifdef OFFLINE_SUPPORTED
+	if (cbex->priv->mode == CAL_MODE_REMOTE) {
+		hier = e_folder_exchange_get_hierarchy (cbex->folder);
+		//cbex->priv->email_address = g_strdup (hier->owner_email);
+		return hier->owner_email;
+	}
+	else
+		return hier->owner_email; /* FIXME : Read this from gconf */
+#else	
 	hier = e_folder_exchange_get_hierarchy (cbex->folder);
-	return hier->owner_email;
+	return hier->owner_email; /* FIXME : Read this from gconf */
+#endif
 }
 
 const char *
@@ -1130,7 +1211,7 @@ check_change_type (gpointer key, gpointer value, gpointer data)
 	if (ecomp != NULL) {
 
 		comp = e_cal_component_new ();
-		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->comp));
+		e_cal_component_set_icalcomponent (comp, ecomp->icomp);
 
 		calobj = e_cal_component_get_as_string (comp);
 		switch (e_xmlhash_compare (change_data->ehash, uid, calobj)){
@@ -1266,7 +1347,7 @@ free_exchange_comp (gpointer value)
 	g_free (ecomp->href);
 	g_free (ecomp->lastmod);
 
-	icalcomponent_free (ecomp->comp);
+	icalcomponent_free (ecomp->icomp);
 
 	for (inst = ecomp->instances; inst; inst = inst->next)
 		icalcomponent_free (inst->data);
