@@ -32,6 +32,7 @@
 #include "exchange-hierarchy-favorites.h"
 #include "exchange-hierarchy-foreign.h"
 #include "exchange-hierarchy-gal.h"
+#include "exchange-offline-listener.h"
 #include "e-folder-exchange.h"
 #include "e2k-autoconfig.h"
 #include "e2k-encoding-utils.h"
@@ -46,6 +47,10 @@
 #include <gal/util/e-util.h>
 #include <libgnome/gnome-util.h>
 
+#include <glade/glade-xml.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtklabel.h>
+
 #include <dirent.h>
 #include <stdlib.h>
 #include <string.h>
@@ -55,6 +60,8 @@
 #define ONE_HUNDRED_NANOSECOND 0.000000100
 #define SECONDS_IN_DAY 86400
 #define PASSWD_EXPIRY_NOTIFICATION_PERIOD 7
+#define FILENAME CONNECTOR_GLADEDIR "/exchange-passwd-expiry.glade"
+#define ROOTNODE "passwd_exp_dialog"
 
 struct _ExchangeAccountPrivate {
 	E2kContext *ctx;
@@ -62,7 +69,7 @@ struct _ExchangeAccountPrivate {
 	GHashTable *standard_uris;
 
 	GMutex *connect_lock;
-	gboolean connecting, connected;
+	gboolean connecting, connected, account_online;
 
 	GPtrArray *hierarchies;
 	GHashTable *hierarchies_by_folder, *foreign_hierarchies;
@@ -150,6 +157,7 @@ init (GObject *object)
 	account->priv->foreign_hierarchies = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->folders = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->discover_data_lock = g_mutex_new ();
+	account->priv->account_online = TRUE;
 }
 
 static void
@@ -885,6 +893,48 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 	return (result == E2K_KERBEROS_PASSWORD_EXPIRED);
 }
 
+static void
+change_passwd_cb (GtkWidget *button, char *old_passwd)
+{
+	gtk_widget_hide (gtk_widget_get_toplevel(button));
+	exchange_get_new_password (old_passwd, TRUE);
+}
+
+static void
+display_passwd_expiry_message (int max_passwd_age, char *old_passwd)
+{
+	GladeXML *xml;
+	GtkWidget *top_widget, *change_passwd_button;
+	GtkResponseType response;
+	GtkLabel *warning_msg_label;
+	char *passwd_expiry_msg = 
+		g_strdup_printf ("Your password will expire in next %d days\n",
+				  max_passwd_age);
+
+	xml = glade_xml_new (FILENAME, ROOTNODE, NULL);
+	g_return_if_fail (xml != NULL);
+	top_widget = glade_xml_get_widget (xml, ROOTNODE);
+	g_return_if_fail (top_widget != NULL);
+
+	warning_msg_label = GTK_LABEL (glade_xml_get_widget (xml, 
+						"passwd_exp_label"));
+	gtk_label_set_text (warning_msg_label, passwd_expiry_msg);
+
+	change_passwd_button = glade_xml_get_widget (xml, 
+						"change_passwd_button");
+	gtk_widget_set_sensitive (change_passwd_button, TRUE);
+	g_signal_connect (change_passwd_button, 
+			  "clicked", 
+			  G_CALLBACK (change_passwd_cb), 
+			  old_passwd);
+
+	response = gtk_dialog_run (GTK_DIALOG (top_widget));
+
+	gtk_widget_destroy (top_widget);
+	g_object_unref (xml);
+	g_free (passwd_expiry_msg);
+}
+
 static void
 find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 {
@@ -892,7 +942,7 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 	int max_pwd_age_days;
 	E2kOperation gcop;
 	E2kGlobalCatalogStatus gcstatus;
-	const char *passwd_expiry_msg=NULL;
+	char *current_passwd;
 
 	/* Check for password expiry period */ 
 	/* This needs to be invoked after is_password_expired(), i.e., 
@@ -924,14 +974,14 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 	if (max_pwd_age > 0) {
 		/* Calculate password expiry period */
 		max_pwd_age_days = 
-			( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
+		( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
 
 		if (max_pwd_age_days <= PASSWD_EXPIRY_NOTIFICATION_PERIOD) {
-			passwd_expiry_msg = 
-			g_strdup_printf ("Your password will expire in next %d days \n", 
-					  max_pwd_age_days);
-			e_notice (NULL, GTK_MESSAGE_WARNING, passwd_expiry_msg); 
-		} 
+			/* This we need for changing password */
+			current_passwd = exchange_account_get_password (account);
+			display_passwd_expiry_message (max_pwd_age_days, 
+						       current_passwd);
+		}
 	} 
 }
 
@@ -989,6 +1039,59 @@ exchange_account_set_password (ExchangeAccount *account, char *old_pass, char *n
 	}
 }
 
+#ifdef OFFLINE_SUPPORTED
+/**
+ * exchange_account_set_offline:
+ * @account: an #ExchangeAccount
+ *
+ * This nullifies the connection and sets the account as offline.
+ * The caller should take care that the required data is fetched
+ * before calling this method.
+ *
+ * Return value: Returns TRUE is successfully sets the account to
+ * offline or FALSE if failed
+ **/
+gboolean
+exchange_account_set_offline (ExchangeAccount *account)
+{
+		
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
+
+	if (account->priv->ctx) {
+		g_object_unref (account->priv->ctx);
+		account->priv->ctx = NULL;
+	}
+
+	account->priv->account_online = FALSE;
+	return TRUE;
+}
+
+/**
+ * exchange_account_set_online:
+ * @account: an #ExchangeAccount
+ *
+ * This nullifies the connection and sets the account as offline.
+ * The caller should take care that the required data is fetched
+ * before calling this method.
+ *
+ * Return value: Returns TRUE is successfully sets the account to
+ * offline or FALSE if failed
+ **/
+gboolean
+exchange_account_set_online (ExchangeAccount *account)
+{
+		
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
+
+	account->priv->account_online = TRUE;
+
+	if (exchange_account_connect (account))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+#endif
 /**
  * exchange_account_connect:
  * @account: an #ExchangeAccount
@@ -1027,8 +1130,11 @@ exchange_account_connect (ExchangeAccount *account)
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
 	g_mutex_lock (account->priv->connect_lock);
-
+#ifdef OFFLINE_SUPPORTED
+	if ((account->priv->connecting) || (!account->priv->account_online)){
+#else
 	if (account->priv->connecting) {
+#endif
 		g_mutex_unlock (account->priv->connect_lock);
 		return NULL;
 	} else if (account->priv->ctx) {
@@ -1330,6 +1436,7 @@ exchange_account_connect (ExchangeAccount *account)
 	}
 	
 	account->priv->connected = TRUE;
+	account->priv->account_online = TRUE;
 	account->priv->connecting = FALSE;
 
 	g_signal_connect (account->priv->ctx, "redirect",
