@@ -335,6 +335,9 @@ create_folder (ExchangeHierarchy *hier, EFolder *parent,
 static ExchangeAccountFolderResult
 remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 {
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type, *physical_uri;
 
@@ -346,27 +349,27 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
         if (strcmp (folder_type, "calendar") == 0) {
                 cal_source_list = e_source_list_new_for_gconf (
                                         gconf_client_get_default (),
-                                        CONF_KEY_CAL);
-                remove_esource (hier->account, EXCHANGE_CALENDAR_FOLDER, 
-				physical_uri, &cal_source_list, FALSE);
+                                        conf_key_cal);
+                remove_esource (hier->account, conf_key_cal, physical_uri,
+                                &cal_source_list, FALSE, FALSE);
                 e_source_list_sync (cal_source_list, NULL);
                 g_object_unref (cal_source_list);
         }
         else if (strcmp (folder_type, "tasks") == 0) {
                 task_source_list = e_source_list_new_for_gconf (
                                         gconf_client_get_default (),
-                                        CONF_KEY_TASKS);
-                remove_esource (hier->account, EXCHANGE_TASKS_FOLDER, 
-				physical_uri, &task_source_list, FALSE);
+                                        conf_key_tasks);
+                remove_esource (hier->account, conf_key_tasks, physical_uri,
+                                &task_source_list, FALSE, FALSE);
                 e_source_list_sync (task_source_list, NULL);
                 g_object_unref (task_source_list);
         }
         else if (strcmp (folder_type, "contacts") == 0) {
                 cont_source_list = e_source_list_new_for_gconf (
                                         gconf_client_get_default (),
-                                        CONF_KEY_CONTACTS);
-                remove_esource (hier->account, EXCHANGE_CONTACTS_FOLDER, 
-				physical_uri, &cont_source_list, FALSE);
+                                        conf_key_contacts);
+                remove_esource (hier->account, conf_key_contacts, physical_uri,
+                                        &cont_source_list, FALSE, TRUE);
                 e_source_list_sync (cont_source_list, NULL);
                 g_object_unref (cont_source_list);
 	}
@@ -432,6 +435,9 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 	const char *folder_type = NULL;
 	const char *physical_uri = NULL;
 	char *new_folder_name;
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 
 	result =  create_internal (hier, hier->toplevel, folder_name, NULL, folder);
@@ -447,12 +453,13 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 		!(strcmp (folder_type, "calendar/public"))) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_CAL);
+						conf_key_cal);
 			add_esource (hier->account,
-				     EXCHANGE_CALENDAR_FOLDER,
+				     conf_key_cal,
 				     new_folder_name,
 				     physical_uri,
-				     &cal_source_list);
+				     &cal_source_list,
+				     FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 
@@ -461,12 +468,13 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 			 !(strcmp (folder_type, "tasks/public"))) {
 				task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_TASKS);
+						conf_key_tasks);
 				add_esource (hier->account,
-				     EXCHANGE_TASKS_FOLDER,
+				     conf_key_tasks,
 				     new_folder_name,
 				     physical_uri,
-				     &task_source_list);
+				     &task_source_list,
+				     FALSE);
 				e_source_list_sync (task_source_list, NULL);
 				g_object_unref (task_source_list);
 
@@ -476,13 +484,14 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 			 !(strcmp (folder_type, "contacts/ldap"))) {
 				cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_CONTACTS);
+						conf_key_contacts);
 
 				add_esource (hier->account,
-				     EXCHANGE_CONTACTS_FOLDER,
+				     conf_key_contacts,
 				     new_folder_name,
 				     physical_uri,
-				     &cont_source_list);
+				     &cont_source_list,
+				     TRUE);
 				e_source_list_sync (cont_source_list, NULL);
 				g_object_unref (cont_source_list);
 		}
