@@ -35,9 +35,10 @@
 #include "e2k-uri.h"
 #include "e2k-utils.h"
 #include "exchange-config-listener.h"
+#include "exchange-folder-size.h"
 
 #include <e-util/e-passwords.h>
-#include <e-util/e-path.h>
+#include "e2k-path.h"
 #include <libedataserver/e-source-list.h>
 
 #include <stdlib.h>
@@ -48,6 +49,7 @@ struct _ExchangeHierarchyWebDAVPrivate {
 	GHashTable *folders_by_internal_path;
 	gboolean deep_searchable;
 	char *trash_path;
+	ExchangeFolderSize *foldersize;
 };
 
 #define PARENT_TYPE EXCHANGE_TYPE_HIERARCHY
@@ -55,6 +57,7 @@ static ExchangeHierarchyClass *parent_class = NULL;
 
 static void folder_type_map_init (void);
 
+static void dispose (GObject *object);
 static void finalize (GObject *object);
 static gboolean is_empty (ExchangeHierarchy *hier);
 static void rescan (ExchangeHierarchy *hier);
@@ -88,6 +91,7 @@ class_init (GObjectClass *object_class)
 	parent_class = g_type_class_ref (PARENT_TYPE);
 
 	/* virtual method override */
+	object_class->dispose = dispose;
 	object_class->finalize = finalize;
 
 	exchange_hierarchy_class->is_empty = is_empty;
@@ -105,6 +109,7 @@ init (GObject *object)
 
 	hwd->priv = g_new0 (ExchangeHierarchyWebDAVPrivate, 1);
 	hwd->priv->folders_by_internal_path = g_hash_table_new (g_str_hash, g_str_equal);
+	hwd->priv->foldersize = exchange_folder_size_new ();
 
 	g_signal_connect (object, "new_folder",
 			  G_CALLBACK (hierarchy_new_folder), NULL);
@@ -112,6 +117,19 @@ init (GObject *object)
 			  G_CALLBACK (hierarchy_removed_folder), NULL);
 }
 
+static void
+dispose (GObject *object)
+{
+	ExchangeHierarchyWebDAV *hwd = EXCHANGE_HIERARCHY_WEBDAV (object);
+
+	if (hwd->priv->foldersize) {
+		g_object_unref (hwd->priv->foldersize);
+		hwd->priv->foldersize = NULL;
+	}
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static void
 finalize (GObject *object)
 {
@@ -162,15 +180,11 @@ static void
 hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 		      gpointer user_data)
 {
-	const char *internal_uri = e_folder_exchange_get_internal_uri (folder);
+	const char *internal_uri ;
 	char *mf_path;
-
-	/* This should ideally not be needed. But, this causes a problem when the
-	server has identical folder names [ internal_uri ] for folders. Very much
-	possible in the case of favorite folders */
-	if (g_hash_table_lookup (EXCHANGE_HIERARCHY_WEBDAV (hier)->priv->folders_by_internal_path,
-				(char *)e2k_uri_path (internal_uri)))
-		return;
+	
+	g_return_if_fail (E_IS_FOLDER (folder));
+	internal_uri = e_folder_exchange_get_internal_uri (folder);
 
 	g_hash_table_insert (EXCHANGE_HIERARCHY_WEBDAV (hier)->priv->folders_by_internal_path,
 			     (char *)e2k_uri_path (internal_uri), folder);
@@ -349,6 +363,9 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 	E2kHTTPStatus status;
 	EFolder *dest;
 	char *permanent_url = NULL, *physical_uri;
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type = NULL;
 	ExchangeAccountFolderResult ret_code;
@@ -404,10 +421,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 		    (strcmp (folder_type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_CAL);
-			remove_esource (hier->account, EXCHANGE_CALENDAR_FOLDER,
+						conf_key_cal);
+			remove_esource (hier->account, conf_key_cal,
 					physical_uri, &cal_source_list,
-					FALSE);
+					FALSE, FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -415,10 +432,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 			 (strcmp (folder_type, "tasks/public") == 0)){
 			task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_TASKS);
-			remove_esource (hier->account, EXCHANGE_TASKS_FOLDER,
+						conf_key_tasks);
+			remove_esource (hier->account, conf_key_tasks,
 					physical_uri, &task_source_list,
-					FALSE);
+					FALSE, FALSE);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -426,10 +443,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 			 (strcmp (folder_type, "contacts/public") == 0)) {
 			cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_CONTACTS);
-			remove_esource (hier->account, EXCHANGE_CONTACTS_FOLDER,
+						conf_key_contacts);
+			remove_esource (hier->account, conf_key_contacts,
 					physical_uri, &cont_source_list, 
-					FALSE);
+					FALSE, TRUE);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
@@ -442,7 +459,14 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 static void
 add_href (gpointer path, gpointer folder, gpointer hrefs)
 {
-	if (!strcmp (e_folder_get_type_string (folder), "noselect"))
+	const char *folder_type;
+	
+	folder_type = e_folder_get_type_string (folder);
+
+	if (!folder_type)
+		return;
+	
+	if (!strcmp (folder_type, "noselect"))
 		return;
 
 	g_ptr_array_add (hrefs, path);
@@ -513,6 +537,12 @@ exchange_hierarchy_webdav_status_to_folder_result (E2kHTTPStatus status)
 		return EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR;
 }
 
+ExchangeFolderSize *
+exchange_hierarchy_webdav_get_folder_size (ExchangeHierarchyWebDAV *hwd)
+{
+	return hwd->priv->foldersize;
+}
+
 EFolder *
 exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 					EFolder *parent,
@@ -520,10 +550,13 @@ exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 {
 	EFolder *folder;
 	ExchangeFolderType *folder_type;
-	const char *name, *prop, *outlook_class, *permanenturl;
+	const char *name, *prop, *outlook_class, *permanenturl, *folder_size;
 	int unread;
 	gboolean hassubs;
 
+	g_return_val_if_fail (EXCHANGE_IS_HIERARCHY_WEBDAV (hwd), NULL);
+	g_return_val_if_fail (E_IS_FOLDER (parent), NULL);
+	
 	/* It's possible to have a localized inbox folder named, eg,
 	 * "Innboks", with children whose URIs go through "Inbox"
 	 * instead. (See bugzilla 27065.) This is probably related to
@@ -560,6 +593,10 @@ exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 	permanenturl = e2k_properties_get_prop (result->props,
 						E2K_PR_EXCHANGE_PERMANENTURL);
 
+	folder_size = e2k_properties_get_prop (result->props,
+						E2K_PR_EXCHANGE_FOLDER_SIZE);
+	// Check for errors
+
 	folder = e_folder_webdav_new (EXCHANGE_HIERARCHY (hwd),
 				      result->href, parent,
 				      name, folder_type->component,
@@ -569,8 +606,10 @@ exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 		e_folder_set_custom_icon (folder, "stock_delete");
 	if (hassubs)
 		e_folder_exchange_set_has_subfolders (folder, TRUE);
-	if (permanenturl)
+	if (permanenturl) {
 		e_folder_exchange_set_permanent_uri (folder, permanenturl);
+		exchange_folder_size_update (hwd->priv->foldersize, permanenturl, name, folder_size);
+	}
 
 	return folder;
 }
@@ -580,6 +619,7 @@ static const char *folder_props[] = {
 	E2K_PR_HTTPMAIL_UNREAD_COUNT,
 	E2K_PR_DAV_DISPLAY_NAME,
 	E2K_PR_EXCHANGE_PERMANENTURL,
+	E2K_PR_EXCHANGE_FOLDER_SIZE,
 	E2K_PR_DAV_HAS_SUBS
 };
 static const int n_folder_props = sizeof (folder_props) / sizeof (folder_props[0]);
@@ -645,11 +685,13 @@ scan_offline_cb (const char *physical_path, const char *path, gpointer data)
 	struct scan_offline_data *sod = data;
 	EFolder *folder;
 	char *mf_name;
+	
 
 	mf_name = g_build_filename (physical_path, "connector-metadata.xml", NULL);
 	folder = e_folder_exchange_new_from_file (sod->hier, mf_name);
 	if (!folder) {
 		unlink (mf_name);
+		g_free (mf_name);
 		if (!sod->badpaths)
 			sod->badpaths = g_ptr_array_new ();
 		g_ptr_array_add (sod->badpaths, g_strdup (path));
@@ -694,7 +736,7 @@ exchange_hierarchy_webdav_offline_scan_subtree (ExchangeHierarchy *hier,
 	dir = e_path_to_physical (hier->account->storage_dir, prefix);
 	g_free (prefix);
 	e_path_find_folders (dir, scan_offline_cb, &sod);
-
+	
 	if (sod.badpaths) {
 		for (i = 0; i < sod.badpaths->len; i++) {
 			e_path_rmdir (dir, sod.badpaths->pdata[i]);
